package com.xkrs.dao;

import com.xkrs.model.entity.MemberProject;
import com.xkrs.model.vo.MemberApplicationVo;
import com.xkrs.model.vo.MemberJoinVo;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @author XinYi Song
 */
@Component
public interface MemberProjectDao extends JpaRepository<MemberProject,Long>, JpaSpecificationExecutor<MemberProject> {

    /**
     * 修改成员加入该项目的时间
     * @param joinTime
     * @param projectNumber
     */
    @Query(value = "update member_project set join_time = ?1 where member_project_number = ?2",nativeQuery = true)
    @Modifying(clearAutomatically=true)
    void updateJoinTime(String joinTime,String projectNumber);

    /**
     * 修改成员的申请状态
     * @param applicationType
     * @param projectNumber
     */
    @Query(value = "update member_project set application_type = ?1 where member_project_number = ?2",nativeQuery = true)
    @Modifying(clearAutomatically=true)
    void updateApplicationType(String applicationType,String projectNumber);

    /**
     * 根据用户id和项目编号查询信息
     * @param userId
     * @param projectNumber
     * @param applicationType
     * @return
     */
    MemberProject findByMemberIdAndMemberProjectNumberAndApplicationType(Integer userId,String projectNumber,String applicationType);

    /**
     * 根据用户id和项目编号删除该条信息
     * @param userId
     * @param projectNumber
     */
    @Modifying(clearAutomatically=true)
    void deleteByMemberIdAndMemberProjectNumber(Integer userId,String projectNumber);

    /**
     * 查询成员的加入申请
     * @param projectNumber
     * @return
     */
    @Query(value = "select new com.xkrs.model.vo.MemberApplicationVo (mp.memberId,su.reallyName,mp.applicationTime,mp.applicationType) " +
            "from SysUserEntity su,MemberProject mp where mp.memberId = su.id and mp.memberProjectNumber = :projectNumber")
    List<MemberApplicationVo> selectMemberApplication(@Param("projectNumber") String projectNumber);

    /**
     * 查询加入的成员的信息
     * @param projectNumber
     * @return
     */
    @Query(value = "select new com.xkrs.model.vo.MemberJoinVo (mp.memberId,su.reallyName,mp.joinTime) " +
            "from SysUserEntity su,MemberProject mp where mp.memberId = su.id and mp.applicationType = '1' " +
            "and mp.memberProjectNumber = :projectNumber")
    List<MemberJoinVo> selectMemberJoin(@Param("projectNumber") String projectNumber);

    /**
     * 根据项目id修改用户id
     * @param userId
     * @param projectNumber
     * @param memberId
     */
    @Query(value = "update member_project set member_id = ?1 where member_project_number = ?2 and member_id = ?3",nativeQuery = true)
    @Modifying(clearAutomatically=true)
    void updateMemberIdByProjectNumber(Integer userId,String projectNumber,Integer memberId);

    /**
     * 查询参与的项目的编号
     * @param userId
     * @param applicationType
     * @return
     */
    @Query(value = "select member_project_number from member_project where member_id = ?1 and application_type = ?2",nativeQuery = true)
    List<String> selectProjectNumberByMemberId(Integer userId,String applicationType);

}
