package com.xkrs.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.xkrs.common.encapsulation.PromptMessageEnum;
import com.xkrs.common.tool.JwtUtil;
import com.xkrs.dao.AppletsUserDao;
import com.xkrs.model.entity.AppletsUser;
import com.xkrs.service.AppletsUserService;
import com.xkrs.utils.DecodeUtils;
import com.xkrs.utils.HttpClientUtil;
import io.micrometer.core.instrument.util.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.transaction.Transactional;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

import static com.xkrs.common.encapsulation.OutputEncapsulation.outputEncapsulationObject;

/**
 * @author HP
 */
@Service
public class AppletsUserServiceImpl implements AppletsUserService {
    public static Logger log = LoggerFactory.getLogger(AppletsUserServiceImpl.class);

    @Resource
    private AppletsUserDao appletsUserDao;

    @Resource
    private DecodeUtils decodeUtils;

    /**
     * 用户登录
     * @param avatarUrl  头像
     * @param nickName   昵称
     * @param sex        性别
     * @param code       wx.login()获取的
     * @return
     */
    @Transactional(rollbackOn = Exception.class)
    @Override
    public String userLogin(String avatarUrl, String nickName, Integer sex, String code) {
        Locale locale = LocaleContextHolder.getLocale();
        try {
            log.info("进入微信登录实现方法----------------->");
            String sexs = sex + "";
            // 创建Httpclient对象
            String requestUrl = "https://api.weixin.qq.com/sns/jscode2session";
            Map<String, String> requestUrlParam = new HashMap<>();
            //小程序appId
            requestUrlParam.put("appid","wxcdd789a27dd7dace");
            //小程序secret
            requestUrlParam.put("secret","78436fda2488c08d4a3744bfd1613265");
            //小程序端返回的code
            requestUrlParam.put("js_code",code);
            //默认参数
            requestUrlParam.put("grant_type","authorization_code");
            JSONObject jsonObject = JSON.parseObject(HttpClientUtil.doPost(requestUrl, requestUrlParam));
            // 解析json
            //JSONObject jsonObject = (JSONObject) JSONObject.parse(resultString);
            String sessionKeys = jsonObject.get("session_key").toString();
            log.info("sessionKey-------"+sessionKeys);
            String openId = jsonObject.get("openid").toString();
            //校验openId是否有效
            if (StringUtils.isBlank(openId) || StringUtils.isBlank(sessionKeys)) {
                return outputEncapsulationObject(PromptMessageEnum.PROCESS_FAIL,"用户登陆失败",locale);
            }
            //以unionId去查是否存在用户,如果存在修改用户信息并返回
            AppletsUser appletsUser = appletsUserDao.findByOpenId(openId);
            if (appletsUser != null) {
                appletsUser.setSessionKey(sessionKeys);
                appletsUserDao.updateSessionKey(appletsUser.getId(),sessionKeys);
                //设置token
                log.info("====="+appletsUser.getId());
                //String token = JwtUtil.sign(appletsUser.getNickName(), sessionKeys);
                Map map = new HashMap(3);
                //map.put("token",token);
                map.put("openId",appletsUser.getOpenId());
                map.put("session_key",sessionKeys);
                return outputEncapsulationObject(PromptMessageEnum.SUCCESS,map,locale);
            }else{
                AppletsUser appletsUser1 = new AppletsUser();
                appletsUser1.setNickName(nickName);
                appletsUser1.setUserPhoto(avatarUrl);
                appletsUser1.setUserSex(sexs);
                appletsUser1.setOpenId(openId);
                appletsUser1.setSessionKey(sessionKeys);
                //新增用户信息
                if (appletsUserDao.save(appletsUser1) != null) {
                    log.info("插入用户成功------------>");
                    System.out.println("拿到userId----------------------->"+appletsUser1.getId());
                    //设置token
                    //String token = JwtUtil.sign(appletsUser1.getNickName(), appletsUser1.getSessionKey());
                    Map map = new HashMap(3);
                    //map.put("token",token);
                    map.put("openId",appletsUser1.getOpenId());
                    map.put("session_key",appletsUser1.getSessionKey());
                    return outputEncapsulationObject(PromptMessageEnum.SUCCESS,map,locale);
                } else {
                    return outputEncapsulationObject(PromptMessageEnum.PROCESS_FAIL,"微信登录失败,请稍后重试",locale);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return outputEncapsulationObject(PromptMessageEnum.PROCESS_FAIL,"微信登录失败,请稍后重试",locale);
    }

    /**
     * 解析手机号
     * @param encryptedData
     * @param iv
     * @param openId
     * @return
     */
    @Transactional(rollbackOn = Exception.class)
    @Override
    public String decodePhone(String encryptedData, String iv, String openId) {
        Locale locale = LocaleContextHolder.getLocale();
        log.info("进入解密手机号实现方法------------->");
        try {
            //通过userId获取用户信息用于解密手机号
            AppletsUser appletsUser = appletsUserDao.findByOpenId(openId);
            if (null == appletsUser) {
                return outputEncapsulationObject(PromptMessageEnum.DATA_WRONG,"用户信息错误,请稍后重试",locale);
            }
            if (StringUtils.isNotBlank(appletsUser.getUserPhone())) {
                //以userId作为redisKey存入redis并设置有效时间为半小时
                //redisClient.set(tourismUser.getTourismId(), tourismUser.getUserPhone(), 1800);
                return outputEncapsulationObject(PromptMessageEnum.SUCCESS,"获取用户手机号成功",locale);
            }
            // 用于解密手机号的sessionKey
            String sessionKey = appletsUser.getSessionKey();
            //解密获得手机号
            JSONObject jsonObject = decodeUtils.wxDecrypt(encryptedData, sessionKey, iv);
            String phone = jsonObject.getString("phoneNumber");
            if (StringUtils.isNotEmpty(phone)) {
                log.info("用户手机号为--------->" + phone);
                //插入用户手机号
                appletsUserDao.updatePhone(openId,phone);
                log.info("插入用户手机号成功----------->");
                //redisClient.set(tourismUser.getTourismId(), phone, 1800);
                return outputEncapsulationObject(PromptMessageEnum.SUCCESS,"获取用户手机号成功",locale);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return outputEncapsulationObject(PromptMessageEnum.PROCESS_FAIL,"获取手机号失败",locale);
    }
}
